from typing import Optional
import typer
from corrlib import __app_name__, __version__
from .initialization import create
from .toml import import_toml, update_project
from .find import find_record, find_project, list_projects
from .tools import str2list
import os

app = typer.Typer()


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()


@app.command()
def update(
    path: str = typer.Option(
        str('./corrlib'),
        "--dataset",
        "-d",
    ),
    uuid: str = typer.Argument(),
) -> None:
    """
    Update a project by it's UUID.
    """
    update_project(path, uuid)
    return

@app.command()
def list(
    path: str = typer.Option(
        str('./corrlib'),
        "--dataset",
        "-d",
    ),
    entities: str = typer.Argument('ensembles'),
) -> None:
    """
    List entities. (ensembles, projects)
    """
    if entities in ['ensembles', 'Ensembles','ENSEMBLES']:
        print("Ensembles:")
        for item in os.listdir(path + "/archive"):
            if os.path.isdir(os.path.join(path + "/archive", item)):
                print(item)
    elif entities == 'projects':
        results = list_projects(path)
        print("Projects:")
        header = "UUID".ljust(37) + "| Names"
        print(header)
        for project in results:
            print(project[0], "|", " | ".join(str2list(project[1])))
    return


@app.command()
def find(
    path: str = typer.Option(
        str('./corrlib'),
        "--dataset",
        "-d",
    ),
    ensemble: str = typer.Argument(),
    corr: str = typer.Argument(),
) -> None:
    """
    Find a record in the backlog at hand. Through specifying it's ensemble and the measured correlator.
    """
    results = find_record(path, ensemble, corr)
    print(results)


@app.command()
def importer(
    path: str = typer.Option(
        str('./corrlib'),
        "--dataset",
        "-d",
    ),
    file: str = typer.Argument(
    ),
    copy_file: bool = typer.Option(
        bool(True),
        "--save",
        "-s",
    ),
) -> None:
    """
    Import a project from a .toml-file via CLI.
    """
    import_toml(path, file, copy_file)
    return


@app.command()
def init(
    path: str = typer.Option(
        str('./corrlib'),
        "--dataset",
        "-d",
    ),
) -> None:
    """
    Initialize a new backlog-database.
    """
    create(path)
    return


@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit.",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return
